import {EventEmitter,
        Output,
        Input}              from '@angular/core';
import {FormGroup}          from '@angular/forms';

import {BaseFilter}         from '../shared/base.filter';
import {CustomValidators}   from '../../shared/service/custom-validators.service';

export abstract class BaseFilterComponent {
    filter: BaseFilter;
    myForm: FormGroup;                  // form de búsqueda
    filterOrders: FilterOrder[];        // (opcional) posibles ordenaciones para el listado de resultados
    currentOrder: string;               // identificador del orden actual
    @Input() displayFilter: boolean;    // nos indica si debemos o no mostrarnos    
    @Output() onSearch = new EventEmitter<BaseFilter>(); // se activará cuando pulsen en buscar
    @Output() onReset = new EventEmitter<BaseFilter>();

    constructor(){
    }

    abstract createNewFilter(): BaseFilter;

    submit(value?: string): void {
        if (!this.myForm.valid) { //this.myForm.dirty puede que no lo hayamos tocado
            let errores = CustomValidators.validateForm(this.myForm);
            alert(`El filtre de cerca conté les següents errades: ${errores.reduce((v1, v2) => v1 + '\n' + v2, '')}`);
        }else{
            this.filter.pagina = 1; //cada vez que ejecutamos una búsqueda empezamos en la página 1
            this.onSearch.emit(this.filter); //comunicamos hacia arriba que debe realizarse la búsqueda
        }
    }

    resetFilter(): boolean {
        this.filter.reset();
        this.onReset.emit(this.filter);
        this.postResetFilter();
        return false;
    }

    postResetFilter() {
        this.filterOrders && this.filterOrders[0] && this.changeOrder(this.filterOrders[0]);
    }

    changeOrder(newOrder: FilterOrder) {
        if(this.filterOrders) {
            this.filter.columnasOrdenacion = this.filterOrders.find(o => o.value === newOrder.value).order;
            this.currentOrder = newOrder.value;
        }
    }

    get diagnostic() { return JSON.stringify(this.filter); }
}

export class FilterOrder {
    constructor(
        public value: string,
        public label: string,
        public order: string[]
    ) {}
}