import { Subject } from "rxjs";
import { PageStatus } from "../../shared/global.constants";
import { MessageService } from "../../shared/service/message.service";
import { ToolbarService } from "../../shared/service/toolbar.service";

/**
 * Clase base mínima para cualquier componente.
 * @version 2.0
 */
export abstract class BaseComponent {
  protected toolbarService: ToolbarService;
  protected messageService: MessageService; // para mostrar mensajes por pantalla al usuario
  status: PageStatus; // determina en qué estado está la página: PageStatus.Init, PageStatus.SEARCHING, PageStatus.FINISH_SEARCH, PageStatus.ERROR
  componentId: string;
  toolbar: any;
  destroy$: Subject<boolean> = new Subject<boolean>();

  constructor(messageService: MessageService, toolbarService?: ToolbarService, componentId: string = "BASE") {
    this.messageService = messageService;
    this.toolbarService = toolbarService;
    this.componentId = componentId;
    // console.log(`Soy el componente ${componentId}`);
  }

  ngOnDestroy() {
    this.destroy$.next(true);
    this.destroy$.unsubscribe();
  }

  resetData(): void {
    //this.errorMessage = '';
  }

  updateToolbar(): void {
    // console.log(`invocación para updateToolbar del componente ${this.componentId}. Toolbar enviada:`);
    // console.log(this.toolbar);
    this.toolbarService && this.toolbarService.updateToolbar(this.componentId, this.toolbar);
  }

  converToDate(fecha: string): Date {
    return new Date(fecha);
  }
}
