import {Observable} from 'rxjs'; 

import {BaseFilter} from './base.filter';
import {Item}       from '../../shared/data/item';
import {FormType}   from '../../shared/global.constants';

/**
 * Interface básica para cualquier servicio CRUD.
 */
export interface BaseCRUDServiceInterface
{
   /** Inserta o modifica la información del elemento. */
   save(object: Item, mode: FormType, params?:{[key:string]:string});
   
   /** Elimina el elemento. */
   delete(id: Number, params?:{[key:string]:string});

   /** Elimina todos los elementos especificados por parámetros */
   deleteAll(items: number[], params?:{[key:string]:string}): Observable<any>;
   
   /** Recupera la información del elemento. */
   get(id: Number, params?:{[key:string]:string});

   /** Recupera todos los elementos */
   getAll(params?:{[key:string]:string});
   
   /** Recupera la lista (opcionalmente, filtrada y paginada) de elementos. */
   getAllByFilter(filter: BaseFilter, params?:{[key:string]:string});
   
   /** Recupera el número total de la lista (filtrada) de elementos. */
   countAllByFilter(filter: BaseFilter, params?:{[key:string]:string});
} 