/**
 * Actualiza el elemento de la lista en caso de que exista en dicha lista, o lo inserta en caso de que no exista
 * @param element objeto que queremos actualizar o insertar en la lista
 * @param list lista de elementos
 */
export function upsertElement(element, list) {
  let find: boolean = false;
  if (!list) list = [];
  list.forEach((item, index) => {
    if (item.id == element.id) {
      list[index] = element;
      find = true;
    }
  });
  if (!find) {
    list.push(element);
  }
  return list;
}

/**
 * Elimina el elemento de la lista
 * @param element elemento a eliminar de la lista
 * @param list lista de elementos
 */
export function deleteElement(element, list) {
  return list.filter(item => item.id != element.id);
}

/** Permite copiar las propiedades de un objeto exceptuando las propiedades indicadas en keys.
 * Util para obtener los valores del filtro exceptuando las propiedades pagina y resultados.
 * La copia se realiza mediante Object.assign por lo que hay que tenerlo en cuenta por temas de propiedades anidadas
 * @param obj objeto que queremos copiar
 * @param keys array de propiedades que no queremos copiar
 */
export function reject(obj, keys) {
  return Object.keys(obj)
    .filter(k => !keys.includes(k))
    .map(k => Object.assign({}, { [k]: obj[k] }))
    .reduce((res, o) => Object.assign(res, o), {});
}

/** Permite seleccionar las propiedades de un objeto indicadas en keys, excluyendo el resto.
 * La copia se realiza mediante Object.assign por lo que hay que tenerlo en cuenta por temas de propiedades anidadas
 * @param obj objeto que queremos copiar
 * @param keys array de propiedades que queremos copiar
 */
export function choose(obj, keys) {
  return Object.keys(obj)
    .filter(k => keys.includes(k))
    .map(k => Object.assign({}, { [k]: obj[k] }))
    .reduce((res, o) => Object.assign(res, o), {});
}