import { Injectable } from "@angular/core";
import { Observable, Subject } from "rxjs";

import { BaseCRUDService } from "../../base/shared/base.service";
import { UsuarioDTO } from "../../common/dto/usuarioDTO";
import { MessageService } from "../../shared/service/message.service";
import { HttpUtilsService } from "../../shared/service/http-utils.service";

@Injectable()
export class LoginService extends BaseCRUDService {
  protected baseUrl: string = "login";

  private usuario: UsuarioDTO;

  private observableLoginSource = new Subject<UsuarioDTO>();
  public observableLogin;

  constructor(protected messageService: MessageService, protected http: HttpUtilsService) {
    super(messageService, http);

    this.observableLogin = this.observableLoginSource.asObservable();
  }

  setUsuario(usuario: UsuarioDTO) {
    this.usuario = usuario;
    this.observableLoginSource.next(usuario);
  }

  getUsuario(): UsuarioDTO {
    return this.usuario;
  }

  login(): Observable<UsuarioDTO> {
    return this.http.getHttpGet<UsuarioDTO>(this.baseUrl);
  }
}
