import {Component,
        OnInit,
        OnDestroy}              from '@angular/core';
import {FormBuilder,
        Validators}             from '@angular/forms';
import {Router,
        ActivatedRoute}         from '@angular/router';
import {Config}                 from '../shared/data/config';
import {ConfiguracionService}   from '../shared/service/configuracion.service';
import {MessageService}         from '../../shared/service/message.service';
import {ToolbarService}         from '../../shared/service/toolbar.service';
import { BaseFormComponent } from '../../base/component/base-form.component';
import { CustomValidators } from '../../shared/service/custom-validators.service';
import { FormType, PageStatus } from '../../shared';

/**
 * Componente que implementa un formulario para insertar, modificar, eliminar y/o consultar un elemento de tipo Usuario.
 */
@Component({
    selector: 'config-form',
    templateUrl: './config-form.component.html'
})
export class ConfigFormComponent extends BaseFormComponent implements OnInit, OnDestroy {
    object: Config;         // objeto configuracion
    
    private subscribeParams: any;       //nos permitirá recibir los parametros de entrada (URL) a este componente 
    
    constructor(
        protected configService: ConfiguracionService,
        protected router: Router,
        protected activatedRoute: ActivatedRoute,
        protected messageService: MessageService,
        protected toolbarService: ToolbarService,
        private formBuilder: FormBuilder
    ) {
        super(configService, router, activatedRoute, messageService, toolbarService);
        this.object = Config.newObject();
        this.myForm = this.formBuilder.group({
            'codigo': [this.object.codigo, [Validators.required, CustomValidators.etiquetaValidator]],
            'valor': [this.object.valor, []]
        });
        
    }

    /** Inicializa la ficha: lee el elemento con el id indicado e inicializa el componente. */
    ngOnInit() {
        let id;
        this.subscribeParams = this.activatedRoute.params.subscribe(params => {
           this.init(params['id']);
        });
    }

    /** Cierra la ficha: libera memoria, desvincula subscripciones, etc. */
    ngOnDestroy() {
        if (this.subscribeParams){
           this.subscribeParams.unsubscribe(); // necesario para evitar memory leaks
        }
    }

    init(id){
        super.init(id);

        //definimos la miga de pan en el OnInit porque necesitamos saber si estamos editando o creando un unevo usuario
        this.breadcrumb = [];
        this.breadcrumb.push({ label: 'Desenvolupament'});
        this.breadcrumb.push({ label: 'Configuració', url: 'developer/config'});
        if (!id){
            this.breadcrumb.push({ label: 'Nou paràmetre' });
        }else{
            this.breadcrumb.push({ label: "Edició de paràmetre" });
        }
    }

    /** Limpia el formulario. */
    cleanForm(): void {
        super.cleanForm();
        this.object = Config.newObject();
    }

    /** Cierra el formulario y vuelve a la pantalla anterior. */
    goBack(): void {
        this.router.navigate(['developer/config']);
    }

    /** Información de debug. */
    get diagnostic() { return JSON.stringify(this.object); }

}