import {Component}              from '@angular/core';
import {Router,
        ActivatedRoute}         from '@angular/router';
import {ConfiguracionService}   from '../shared/service/configuracion.service';
import {ConfigFilter}           from '../shared/data/config.filter';
import {ToolbarService}         from '../../shared/service/toolbar.service';
import {MessageService}         from '../../shared/service/message.service';
import { BaseListComponent } from '../../base/component/base-list.component';
import { BitToolbarComponent } from '../../shared/component/bit-toolbar.component';

@Component({
    selector: 'config-list',
    template: `
        <bit-header [breadcrumb]="breadcrumb" (onToolbarButtonPressed)="onToolbarButtonPressed($event)"></bit-header>
        <config-list-tableview [items]=items></config-list-tableview> 
        <bit-footer [autoQueryObs]="autoQuerySubject.asObservable()" [totalItems]="totalItems" [status]="status" type="list" [paginator]="true" (onPageChange)="onPageChange($event)" (onNewPageSize)="onNewPageSize($event)"></bit-footer>
    `
})
export class ConfigListComponent extends BaseListComponent {
    // Tenemos una referencia al componente hijo que contiene el filtro 
    //@ViewChild(TipoDatoListFilterComponent) tipoDatoListFilterComponent: TipoDatoListFilterComponent;

    constructor(
        protected configService: ConfiguracionService,
        protected messageService: MessageService,
        protected toolbarService: ToolbarService,
        protected activatedRoute: ActivatedRoute,
        protected router: Router
    ) {
        super(configService, messageService, activatedRoute, router, toolbarService);
        this.breadcrumb.push({label: 'Desenvolupament'});
        this.breadcrumb.push({label: 'Configuració'});
    }

    /** Tenemos que ver si activamos la autobúsqueda entre otras cosas */
    ngOnInit(): any {
        super.init(ConfigFilter.newObject());
        this.toolbar[BitToolbarComponent.NEW_BUTTON].enable=true;
        this.toolbar[BitToolbarComponent.NEW_BUTTON].visible=true;
        this.toolbar[BitToolbarComponent.NEW_BUTTON].estilo="fa fa-database";
        this.updateToolbar();        
        this.search(false); // no hay filtro ni paginación, buscamos nada más entrar
    }

    prepareDataForSubmit(): void { }

}