import {Component}              from '@angular/core';
import {FormBuilder, 
        Validators}             from '@angular/forms';

import {LogExceptionFilter}     from '../shared/data/log-exception.filter'; 
import {MessageService}         from '../../shared/service/message.service';
import {SessionSharedService}   from '../../shared/service/session-shared.service';
import { BaseFilterComponent } from '../../base/component/base-filter.component';
import { BaseFilter } from '../../base/shared/base.filter';
import { CustomValidators } from '../../shared/service/custom-validators.service';

@Component({

    selector: 'exception-list-filter',
    templateUrl: './exception-list-filter.component.html'
})
export class ExceptionListFilterComponent extends BaseFilterComponent {
    filter: LogExceptionFilter;         // contiene el filtro de búsqueda. Será accedido directamente desde el padre

    constructor(
        private formBuilder: FormBuilder, 
        private messageService: MessageService,
        private sessionShared: SessionSharedService
    ) {
        super();
        //gestión del filtro
        this.filter = this.sessionShared.get("EXCEPTION_FILTER");
        if (this.filter === undefined) {
            this.filter = this.createNewFilter();
            this.filter.pagina = BaseFilter.PAGE;
            this.filter.resultados = BaseFilter.RESULTADOS_PAGE;
            this.sessionShared.set("EXCEPTION_FILTER", this.filter);
        }   
        //construimos el form         
        this.myForm = formBuilder.group({
            'fecha': [this.filter.fecha, [Validators.required, CustomValidators.dataValidator]],
            'desde': [this.filter.desde, [Validators.required]],//CustomValidators.timeValidator
            'hasta': [this.filter.hasta, [Validators.required]]
        });
    }

    /** Crea un nuevo filtro para realizar la búsqueda */
    createNewFilter(): LogExceptionFilter {
        return LogExceptionFilter.newObject(); 
    }

}
