import {Component,
        OnInit,
        ViewChild}                      from '@angular/core';
import {ActivatedRoute,
        Router}                         from '@angular/router';
import {ExceptionListFilterComponent}   from './exception-list-filter.component';
import {DeveloperService}               from '../shared/service/developer.service';
import {MessageService}                 from '../../shared/service/message.service';
import {ToolbarService}                 from '../../shared/service/toolbar.service';
import { BaseListComponent } from '../../base/component/base-list.component';

/**
 * Componente que implementa el listado de errores de aplicación para auditoría.
 * Permite consultar las exception que se han producido en la App.
 **/
@Component({

    selector: 'exception-list',
    template: `   
   <bit-header [breadcrumb]=breadcrumb (onToolbarButtonPressed)="onToolbarButtonPressed($event)"></bit-header>
   <exception-list-filter [displayFilter]="true" (onSearch)="onSearch($event)" (onReset)=onReset($event)></exception-list-filter>
   <exception-list-tableview [items]=items></exception-list-tableview>
   <bit-footer [autoQueryObs]="autoQuerySubject.asObservable()" [totalItems]="totalItems" [status]="status" type="list" [paginator]="true" (onPageChange)="onPageChange($event)" (onNewPageSize)="onNewPageSize($event)"></bit-footer>
   `
})
export class ExceptionListComponent extends BaseListComponent implements OnInit {
    // Tenemos una referencia al componente hijo que contiene el filtro
    @ViewChild(ExceptionListFilterComponent) exceptionListFilterComponent: ExceptionListFilterComponent;

    constructor(
        protected developerService: DeveloperService,
        protected messageService: MessageService,
        protected toolbarService: ToolbarService,
        protected activatedRoute: ActivatedRoute,
        protected router: Router
    ) {
        super(developerService, messageService, activatedRoute, router);
        this.breadcrumb.push({label: 'Developer'});
        this.breadcrumb.push({label: 'Exceptions'});
    }

    /** Tenemos que ver si activamos la autobúsqueda entre otras cosas */
    ngOnInit(): any {
        super.init(this.exceptionListFilterComponent.filter);
    }

    /** Preparampos los datos antes de enviarlos al servidor */
    prepareDataForSubmit(): void {
        // nothing to do
    }

}
