import {Component,
        OnInit}           from '@angular/core';
import {FormBuilder,
        FormControl,
        Validators}       from '@angular/forms';
import {HQL}              from '../shared/data/hql';
import {MessageService}   from '../../shared/service/message.service';
import {DeveloperService} from '../shared/service/developer.service';
import { PageStatus } from '../../shared/global.constants';
import { BitMessage } from '../../shared/data/bitmessage';

/**
 * Clase que implementa la consulta de información con consultas HQL.
 */
@Component({

   selector: 'hql',
   templateUrl: './hql.component.html'
})
export class HQLComponent implements OnInit {
   breadcrumb: any = [];
   status: PageStatus;
   messages: Map<PageStatus, string> = new Map<PageStatus, string>();
   hql: HQL;
   hqlControl: FormControl;

   /** Constructor de la clase. */
   constructor(
        private developerService: DeveloperService, 
        private formBuilder: FormBuilder,
        private messageService: MessageService
    ) {}
   
   /** Inicializa la ficha: define la validación. */
   ngOnInit(): any {
      this.init();
      this.hqlControl = this.formBuilder.control('', [Validators.required]);
      this.breadcrumb = [];
      this.breadcrumb.push({label: 'Developer'});
      this.breadcrumb.push({label: 'SQL/HQL'});
      this.messages.set(PageStatus.Init, "Introdueixi el SQL/HQL");
      this.messages.set(PageStatus.Searching, "Executant la sentència");
      this.messages.set(PageStatus.FinishSearch, "Sentència executada correctament");
   }

   /** Inicializa el componente. */
   init() {
      this.status = PageStatus.Init;
      this.hql = HQL.newObject();
      this.hql.reset();
   }

   /** Envía la consulta/ejecución de HQL. */
   sendSHQL(esHQL: boolean, execute: boolean, simulate: boolean) {
      // Evitamos iniciar una nueva búsqueda cuando la anterior aún está en curso (p. ej., double click)
      if (this.status === PageStatus.Searching) { return; }
      this.status = PageStatus.Searching;
      //this._developerService.sendHQLTest()
      this.developerService.sendSHQL(esHQL, this.hql, execute, simulate)
          .subscribe(
            result => { 
                this.hql = result; 
                this.status = PageStatus.FinishSearch; 
            },
            error  => { 
                this.status = PageStatus.Error;
                let titulo = (simulate)?"Resultat de la simulació":"Error";
                let cuerpo = error.text + (error.trazaError?("\nTraça:\n"+error.trazaError):"");
                this.messageService.sendMessage(new BitMessage(titulo, cuerpo));
            }
          );
   }

   /** Limpia la información de consulta/ejecución. */
   resetForm(): void {
      this.init();
   }
   
   /** Flag: la página está realizando la búsqueda. */
   get searchInitiated() { return this.status === PageStatus.Init; }

   /** Flag: la página está realizando la búsqueda. */
   get searching() { return this.status === PageStatus.Searching; }

   /** Flag: la página ha finalizado la última búsqueda. */
   get searchFinished() { return this.status === PageStatus.FinishSearch; }

   /** Información de debug. */
   get diagnostic() { return JSON.stringify(this.hql); }

}