import {Component,
        OnInit}                   from '@angular/core';
import {ActivatedRoute, 
        Router}                   from '@angular/router';
import {LogService}               from '../shared/service/log.service';
import {LogFilter}                from '../shared/data/log.filter';
import {MessageService}           from '../../shared/service/message.service';
import { BaseListComponent } from '../../base/component/base-list.component';

/**
 * Componente que implementa el listado de log para auditoría.
 * Permite consultar todas las acciones que han realizad o los usuarios en la aplicación.
 **/
@Component({

  selector: 'log-list',
  template: `   
   <bit-header [toolbar]=toolbar (onToolbarButtonPressed)="onToolbarButtonPressed($event)"></bit-header>
   <log-list-tableview [items]=items></log-list-tableview>
   <bit-footer [autoQueryObs]="autoQuerySubject.asObservable()" [totalItems]="totalItems" [status]="status"></bit-footer>
   `
})
export class LogListNoFiltroComponent extends BaseListComponent implements OnInit {
  protected filter: LogFilter;
  
  constructor(
    protected logService: LogService,
    protected messageService: MessageService,
    protected activatedRoute: ActivatedRoute,
    protected router: Router
  ) {
    super(logService, messageService,  activatedRoute, router);
  }

  /** Construimos un filtro manual, desactivamos el botón de filtro del toolbar y buscamos 50 elementos sin paginación*/
  ngOnInit(): any {
    this.filter = new LogFilter("","","","",null,false,50,1);
    this.toolbar["filter"].visible = false;
    this.search();
  }

  /** Preparampos los datos antes de enviarlos al servidor */
  prepareDataForSubmit(): void {
    // nothing to do
  }

}
