import {Component,
        OnInit,
        ViewChild}                from '@angular/core';
import {ActivatedRoute,
        Router}                   from '@angular/router';
        
import {LogListFilterComponent}   from './log-list-filter.component';
import {LogService}               from '../shared/service/log.service';
import {MessageService}           from '../../shared/service/message.service';
import {ToolbarService}           from '../../shared/service/toolbar.service';
import { BaseListComponent } from '../../base/component/base-list.component';

/**
 * Componente que implementa el listado de log para auditoría.
 * Permite consultar todas las acciones que han realizad o los usuarios en la aplicación.
 **/
@Component({

    selector: 'log-list',
    template: `   
    <bit-header [breadcrumb]=breadcrumb (onToolbarButtonPressed)="onToolbarButtonPressed($event)"></bit-header>
    <log-list-filter [displayFilter]="true" (onSearch)="onSearch($event)" (onReset)=onReset($event)></log-list-filter>
    <log-list-tableview [items]=items></log-list-tableview>  
    <bit-footer [autoQueryObs]="autoQuerySubject.asObservable()" [totalItems]="totalItems" [status]="status" type="list" [paginator]="true" (onPageChange)="onPageChange($event)" (onNewPageSize)="onNewPageSize($event)"></bit-footer>
    `
})
export class LogListComponent extends BaseListComponent implements OnInit {
    // Tenemos una referencia al componente hijo que contiene el filtro
    @ViewChild(LogListFilterComponent) logListFilterComponent: LogListFilterComponent;

    constructor(
        protected logService: LogService,
        protected messageService: MessageService,
        protected activatedRoute: ActivatedRoute,
        protected router: Router,
        protected toolbarService: ToolbarService
    ) {
        super(logService, messageService, activatedRoute, router, toolbarService);
        this.breadcrumb.push({label: 'Desenvolupament', url: 'developer/exceptions'}); //es un ejemplo
        this.breadcrumb.push({label: 'Log'});
    }

    /** Tenemos que ver si activamos la autobúsqueda entre otras cosas. Pasamos el objeto filtro de nuestro hijo para realizar esa 
     * hipotética autobúsqueda
     */
    ngOnInit(): any {
        super.init(this.logListFilterComponent.filter);
    }

    /** Preparampos los datos antes de enviarlos al servidor */
    prepareDataForSubmit(): void {
        // nothing to do
    }

}
