import {Component, 
    OnDestroy,
    OnInit}         from '@angular/core';
import {Subscription}   from 'rxjs';

import {MessageService} from '../service/message.service';
import {BitMessage}     from '../data/bitmessage';
import {Error}          from '../data/error'; 

@Component({   
selector: 'bit-dialog',     
template: `
  <p-dialog *ngIf="!isSeverityInfo" class="scroll-y scroll-{{message.height}}" width="{{message.width}}" header="{{message.title}}" [(visible)]="display" [modal]="message.modal" showEffect="fade">
     <p class="mensaje-dialogo">{{message.message}}</p>
     <span *ngIf="message.masInfo"><pre>{{message.masInfo}}</pre></span>
     <footer>
        <div class="ui-dialog-buttonpane ui-widget-content ui-helper-clearfix">&nbsp;</div>
     </footer>
  </p-dialog>
  <p-dialog *ngIf="isSeverityInfo && !message.json" class="scroll-y scroll-{{message.height}}" width="{{message.width}}" header="{{message.title}}" [(visible)]="display" [modal]="message.modal" showEffect="fade">
     <p class="mensaje-dialogo" [innerHTML]="message.message"></p>         
  </p-dialog>
  <p-dialog *ngIf="message.json" class="scroll-y scroll-{{message.height}}" width="{{message.width}}" header="{{message.title}}" [(visible)]="display" [modal]="message.modal" showEffect="fade">
     <pre [innerHTML]="syntaxHighlight(message.message)"></pre>         
  </p-dialog>
`
})
export class BitDialogComponent implements OnInit, OnDestroy {
public static MENSAJE_VOID: string = "No hi ha dades per mostrar";
display: boolean = false;
message: BitMessage;
height: number = 200;

private subscriptionMessages: Subscription;    
private subscriptionErrors: Subscription;

constructor(messageService: MessageService) {
    //console.log("DialogComponent contructor(" + messageService  + ")");
    //Nos suscribimos a la espera de que envíen mensajes
    this.subscriptionMessages = messageService.observableMessages.subscribe(
        message => {
            this.showMessage(message);
    });
    this.subscriptionErrors = messageService.observableErrors.subscribe(
        message => {
            this.showError(message);
    });

}

ngOnInit(){
    //inicializamos el objeto para que no pete el template
    this.message = new BitMessage('','');
}

showMessage(message: BitMessage) {      
    this.message=message;
    if (this.message.json){  
        try{                                 
            let json = eval("(" + this.message.message + ")");//{"w":1, "b":'foo', "c":[false,'false',null, 'null', {"d":{"e":1.3e5,"f":'2.3e5'}}]}; 
            this.message.message = JSON.stringify(json, undefined, 2);   
        }catch(error){
            console.warn("No pudo convertirse a JSON la información del log");
        }
    }              
    console.log(this.message.message);
    (message.height === 0)?this.message.height = this.calculateHeightNumber():undefined;
    this.display = true;
}

showError(error: Error) {        
    this.showMessage(new BitMessage(`Error accediendo a ${error.url}`, error.textCA || error.text, 1024, 600, false, BitMessage.ERROR, error.trazaError));
}

get isSeverityInfo() {
    return this.message && this.message.severity === 'info';
}

/**
 * Calcula la altura del diálogo en función de la información a mostrar.
 * TODO: Contar las líneas reales del mensaje en vez de la longitud
 */
private calculateHeightNumber(): number{
    let msj = this.message.message;
    if (msj){
        if (msj.length < 100){
            return 100;
        }else if (msj.length < 200){
            return 200;            
        }else if (msj.length < 300){
            return 300;            
        }else if (msj.length < 400){
            return 400;            
        }else{
            return 500;
        }            
    }else{
        this.message.message = BitDialogComponent.MENSAJE_VOID;
        return 100;
    }
}

ngOnDestroy() {
    // prevent memory leak when component destroyed
    this.subscriptionMessages.unsubscribe();
}

syntaxHighlight(json) {
    if (typeof json != 'string') {
        json = JSON.stringify(json, undefined, 2);
    }
    json = json.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
    return json.replace(/("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^\\"])*"(\s*:)?|\b(true|false|null)\b|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)/g, function (match) {
        var cls = 'number';
        if (/^"/.test(match)) {
            if (/:$/.test(match)) {
                cls = 'key';
            } else {
                cls = 'string';
            }
        } else if (/true|false/.test(match)) {
            cls = 'boolean';
        } else if (/null/.test(match)) {
            cls = 'null';
        }
        return '<span class="' + cls + '">' + match + '</span>';
    });
}

}