import {Component, Inject, Input, Output, EventEmitter} from '@angular/core';
import {Fichero}            from '../data/fichero';
import {BitMessage}         from '../data/bitmessage';
import {UploadService}      from '../service/upload.service';
import {MessageService}     from '../service/message.service';

@Component({
selector: 'bit-form-upload',
template: `
    <form method="post" enctype="multipart/form-data" (submit)="submit()" style="display: inline;">
        <div *ngIf="fileList != null && fileList.length > 0 && showFiles && multiple" class="col-sm-4">
            <a *ngFor="let file of fileList; let i=index;" 
                target="_blank" href="{{downloadUrl(file.id)}}">
                {{file.nombre}}
            </a>
        </div>
        <div *ngIf="simpleFile != null && showFiles && !multiple" class="col-sm-4">
            <a *ngIf="puedeAbrir" target="_blank" href="{{downloadUrl(simpleFile.id)}}">
                {{simpleFile.nombre}}
            </a>
            <span *ngIf="!puedeAbrir">{{simpleFile.nombre}}</span>
        </div>
        <div *ngIf="!soloLectura" class="col-sm-8">
            <bit-upload 
                [uploading]=uploading
                [reset]="resetUpload" 
                (onFileChange)="fileChange($event)" 
                [showFiles]=true 
                [multiple]=multiple 
                [id]="id">
            </bit-upload>
        </div>
    </form>
`
})
export class BitFormUploadComponent {
    @Input() params: {[key:string]:string};
    @Input() showFiles: boolean;
    @Input() id: string = "file";
    @Input() multiple: boolean;
    @Input() searchLabel: string;
    @Input() soloLectura: boolean;
    @Input() simpleFile: Fichero;
    @Input() fileList: Array<Fichero>;
    @Input() puedeAbrir: boolean = true;
    @Output() private onFilesUpload: EventEmitter<Array<Fichero>> = new EventEmitter<Array<Fichero>>();
    uploading: boolean = false;
    private filesToUpload: Array<File>;
    private resetUpload: EventEmitter<any> = new EventEmitter<any>(); // para avisar a bitupload de que resetee su formulario
    private static DOWNLOAD_URL;

    constructor(private uploadService: UploadService,
                protected messageService: MessageService,
                @Inject("DOWNLOAD_URL") private downloadUrlVariable: string) {
        BitFormUploadComponent.DOWNLOAD_URL = downloadUrlVariable;
    }

    ngOnInit() { }

    fileChange(files: Array<File>){
        this.filesToUpload = files;
    }

    // para utilizar desde la propia plantilla
    downloadUrl(id: number) {
        return BitFormUploadComponent.getUrlDownloadFile(id);
    }

    // para utilizar desde otro componente
    static getUrlDownloadFile(id: number){
        return BitFormUploadComponent.DOWNLOAD_URL + 'ficheros/download/'+id;
    }

    submit() {
        console.log("submit upload files");
        this.uploading = true;
        this.uploadService.upload(null, this.filesToUpload, this.params).then((result: Array<Fichero>) => {
            this.uploading = false;
            console.log(result);
            let jsonArray = eval("(" + result + ")");
            //this.messageService.sendNotification(new BitMessage("Informació", "S'ha pujat el fitxer correctament al servidor"));
            this.resetUpload.emit(true);            //comunicamos a bit-upload que resetee la información de los ficheros cargados
            this.onFilesUpload.emit(jsonArray);     //comunicamos hacia arriba el resultado satisfactorio de la operación
            if (this.multiple){
                this.fileList = jsonArray;                
            }else{
                this.simpleFile = jsonArray[0];
            }
        }, (error) => {
            this.uploading = false;
            console.log(error);
            let objError = JSON.parse(error);
            this.messageService.sendNotification(new BitMessage("Error", objError.text, undefined, undefined, undefined, BitMessage.ERROR));
            this.onFilesUpload.emit(null);
            this.fileList = [];
            this.simpleFile = null;
        });
    }

    static createNewParamsObject(tipoFichero: string): {[key:string]:string}{
        let paramsUploadFile = {};                
        paramsUploadFile['tipoFichero'] = tipoFichero;
        return paramsUploadFile;
    }

    static addCarpetaToParamsObject(paramsUploadFile:{[key:string]:string}, carpeta: string): void{
        paramsUploadFile['carpeta'] = carpeta;
    }

    static addIdToParamsObject(paramsUploadFile:{[key:string]:string}, id: string): void{
        paramsUploadFile['id'] = id;
    }
    static addTipoFicheroToParamsObject(paramsUploadFile:{[key:string]:string}, tipoFichero: string): void{
        paramsUploadFile['tipoFichero'] = tipoFichero;
    }
}