import { Component, ChangeDetectorRef, forwardRef } from "@angular/core";
import { Input, Output, EventEmitter } from "@angular/core";
import { FormControl } from "@angular/forms";

import { AyudaService } from "../service/ayuda.service";
import { GeneralUtils } from "../service/general-utils.service";

import { NG_VALUE_ACCESSOR, ControlValueAccessor } from "@angular/forms";

const noop = () => {};

export const CUSTOM_INPUT_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => BitInputComponent),
  multi: true
};

/** TODO: cambiar algunas propiedades que están en castellano como nombre, tipo al inglés: name, type
 * Clase que utiliza un custom componente para input text sencillos
 * Utiliza la interfaz ControlValueAccessor
 *    <bit-input2 [(ngModel)]="filter.fecha"
 *       [nombre]="'fecha2'" [control]="myForm.controls.fecha" formControlName="fecha" [tipo]="'date'" >&nbsp;&nbsp;fecha</bit-input2>
 *    <bit-input2 [(ngModel)]="filter.login"
 *       [nombre]="'login'" [control]="myForm.controls.login" formControlName="login" [tipo]="'text'" >&nbsp;&nbsp;login</bit-input2>
 **/

@Component({
  selector: "bit-input",
  template: `
            <label *ngIf="!hideLabel && tipo!='checkbox' && tipo!='oculto'"><ng-content></ng-content>&nbsp;<span *ngIf="obligatorio" class="obligatorio fa fa-asterisk"></span>&nbsp;&nbsp;<i *ngIf="ayuda" class="btn-ayuda fa fa-question" (click)="showAyuda();"></i></label>
            <p *ngIf="modoLecturaSimple" class="lectura" [id]="nombre">{{value_lectura}}</p>
            <pre *ngIf="modoLecturaMultilinea" class="lectura" [id]="nombre">{{value_lectura}}</pre>
           
            <input *ngIf="tipo=='text' && !tooltiptexto && !readOnly" 
                    type="text" class="form-control" [(ngModel)]="value"  id="{{nombre}}" name="{{nombre}}"
                    (ngModelChange)="changed($event)"
                    [attr.directive]="nombreDirectiva" (blur)="onBlur()" (focus)="onFocus()"
                    [placeholder]="placeholder" />
            <input *ngIf="tipo=='text' && tooltiptexto && !readOnly" 
                    type="text" class="form-control" [(ngModel)]="value"  id="{{nombre}}" name="{{nombre}}" 
                    (ngModelChange)="changed($event)"
                    [attr.directive]="nombreDirectiva" (blur)="onBlur()" (focus)="onFocus()" [pTooltip]="tooltiptexto" [tooltipPosition]="tooltipposicion" />
            <input *ngIf="tipo=='oculto' && !readOnly"
                    type="hidden" [(ngModel)]="value"  id="{{nombre}}" name="{{nombre}}" 
                    (ngModelChange)="changed($event)"
                    [attr.directive]="nombreDirectiva" />
            <textarea pInputTextarea *ngIf="tipo=='textarea' && !readOnly" class="form-control h-100 {{estilo}}" [(ngModel)]="value" id="{{nombre}}" name="{{nombre}}"
                    (ngModelChange)="changed($event)"
                    (blur)="onBlur()" (focus)="onFocus()"></textarea>
            <p-checkbox class="{{estilo}}" 
	    		*ngIf="tipo=='checkbox' && !readOnly" binary="true"
			[(ngModel)]="value" 
                        (ngModelChange)="changed($event)"
			[label]="!hideLabel && label" 
			name="{{nombre}}" 
			id="{{nombre}}"
			[disabled]="disabled"
	    ></p-checkbox>
            <i *ngIf="ayuda && tipo=='checkbox' && !readOnly" (click)="showAyuda();"class="btn-ayuda-checkbox fa fa-question"></i>
            <p-radioButton  class="{{estilo}}" *ngIf="tipo=='radiobutton' && !readOnly" binary="true" [(ngModel)]="value" [label]="!hideLabel && label" name="{{nombre}}" value="{{valueRadioButton}}" id="{{nombre}}"
                    (ngModelChange)="changed($event)" [disabled]="disabled"></p-radioButton>

            <p-calendar *ngIf="tipo=='date' && !readOnly" class="FormControl" [(ngModel)]="value" id="{{nombre}}" name="{{nombre}}"                        
                    (ngModelChange)="changed($event)"
                    dateFormat="dd/mm/yy" placeholder="dd/mm/yyyy" [monthNavigator]="true" [readonlyInput]="calendarReadOnlyInput"
                    [minDate]="fechaMinima" [maxDate]="fechaMaxima"
                    [yearNavigator]="true" yearRange="2000:2030" [locale]="LOCALE_ES"></p-calendar>
            <p-calendar *ngIf="tipo=='datetime' && !readOnly" class="FormControl" [(ngModel)]="value" id="{{nombre}}" name="{{nombre}}"                        
                    (ngModelChange)="changed($event)"
                    dateFormat="dd/mm/yy" placeholder="dd/mm/yyyy hh:mm" [monthNavigator]="true"
                    [minDate]="fechaMinima" [maxDate]="fechaMaxima"
                    [yearNavigator]="true" yearRange="2000:2030" [locale]="LOCALE_ES" [showTime]="true"></p-calendar>
            <span *ngIf="tipo=='date' && !readOnly" class="glyphicon glyphicon-calendar icono-input"></span>
            <span *ngIf="tipo=='datetime' && !readOnly" class="glyphicon glyphicon-calendar icono-input"></span>
            <p-calendar *ngIf="tipo=='time' && !readOnly" class="FormControl" [(ngModel)]="value"   id="{{nombre}}" name="{{nombre}}"                        
                    (ngModelChange)="changed($event)"
                    timeFormat="HH:mm" [timeOnly]="true" placeholder="HH:mm" [locale]="LOCALE_ES"></p-calendar>
            <p-autoComplete *ngIf="tipo=='autocomplete' && !readOnly" 
                    [(ngModel)]="value" 
                    id="{{nombre}}" 
                    name="{{nombre}}" 
                    [placeholder]="placeholder"
                    (ngModelChange)="changed($event)"
		    delay="400" 
                    [immutable]="false"
                    [suggestions]="suggestions" 
                    [field]="suggestionField" 
                    (completeMethod)="search($event)" 
                    (onSelect)="selectValue($event)" 
                    (blur)="onBlur()" 
                    (focus)="onFocus()" 
                    [multiple]="multiple"
            ></p-autoComplete>
            <p-inputSwitch *ngIf="tipo=='switch' && !readOnly" onLabel="Sí" offLabel="No" binary="true" [(ngModel)]="value" (ngModelChange)="changed($event)" name="{{nombre}}" id="{{nombre}}"></p-inputSwitch>
            <control-messages *ngIf="control!=null && !readOnly" [control]="control" [field]="nombre"></control-messages>
`,
  providers: [CUSTOM_INPUT_CONTROL_VALUE_ACCESSOR]
})
export class BitInputComponent implements ControlValueAccessor {
  @Input()
  control: FormControl;
  @Input()
  nombre: any;
  @Input()
  nombreDirectiva: any;
  @Input()
  label: string;
  @Input()
  tipo: String;
  @Input()
  estilo: String;
  @Input()
  tooltiptexto: String;
  @Input()
  tooltipposicion: String;
  @Input()
  ayuda: string; // clave de la ayuda contextual
  @Input()
  suggestions: any[];
  @Input()
  suggestionField: string;
  @Input()
  placeholder: string = "";
  @Input()
  readOnly: boolean = false;
  @Input()
  hideLabel: boolean = false; // forzar que no se reserve espacio para el nombre del campo
  @Input()
  disabled: boolean = false;
  @Input()
  valueRadioButton: any; // value radio button

  // calendarios
  @Input()
  fechaMinima: Date; // fecha mínima seleccionable
  @Input()
  fechaMaxima: Date; // fecha máxima seleccionable
  @Input()
  calendarReadOnlyInput: boolean; // desabilitar edicion por teclado

  @Input()
  instanciaPipe: any;
  @Output()
  completeMethod = new EventEmitter<string>();
  @Output()
  onSelect = new EventEmitter<string>();
  @Output()
  onChange = new EventEmitter<string>();

  @Input()
  private forceObligatorio: boolean = false;
  private innerValue: any = "";
  private onTouchedCallback: () => void = noop;
  private onChangeCallback: (_: any) => void = noop;

  constructor(
    protected ayudaService: AyudaService,
    public changeDetectorRef: ChangeDetectorRef
  ) {}

  //get accessor
  get value(): any {
    return this.innerValue;
  }

  //set accessor including call the onchange callback
  set value(v: any) {
    if (this.tipo === "date") {
      //hacemos una reconversion de la fecha a formato corto. La versión final de Prime nos entrega el formato ISO
      //console.log("Valor para la fecha en formato largo {}",v);
      if (v) {
        v = GeneralUtils.getDateAsString(GeneralUtils.stringToDate(v));
      }
      //console.log("Nuevo valor para la fecha {}",v);
    }
    if (this.tipo === "time") {
      //hacemos una reconversion de la fecha a formato corto. La versión final de Prime nos entrega el formato ISO
      //console.log("Valor para la fecha en formato largo {}",v);
      if (v) {
        v = GeneralUtils.getTimeAsString(GeneralUtils.stringToDate(v));
      }
      //console.log("Nuevo valor para la fecha {}",v);
    }
    /*
    if (this.tipo === "datetime") {
      //hacemos una reconversion de la fecha a formato corto. La versión final de Prime nos entrega el formato ISO
      //console.log("Valor para la fecha en formato largo {}",v);
      if (v) {
        v = GeneralUtils.getDateTimeAsString(v);
      }
      //console.log("Nuevo valor para la fecha {}",v);
    }
    */
    if (v !== this.innerValue) {
      this.innerValue = v;
    }
    this.onChangeCallback(v);
  }

  get value_lectura(): string {
    if (this.value == null) {
      return "";
    } // si es null o undefined (es lo que evalúa el "==null")
    else if (this.tipo === "lectura-date") {
      return GeneralUtils.getDateAsString(
        GeneralUtils.stringToDate(this.value)
      );
    } // valor en formato ISO
    else if (this.tipo === "lectura-time") {
      return GeneralUtils.getTimeAsString(
        GeneralUtils.stringToDate(this.value)
      );
    } // valor en formato ISO
    else if (this.tipo === "lectura-datetime" || this.tipo === "datetime") {
      return GeneralUtils.getDateTimeAsString(
        GeneralUtils.stringToDate(this.value)
      );
    } // valor en formato ISO
    else if (this.value === true) {
      return "Sí";
    } else if (this.value === false) {
      return "No";
    } else {
      return this.value;
    }
  }

  changed(event) {
    this.onTouchedCallback();
    this.onChangeCallback(this.innerValue);
    this.onChange.emit(event);
  }

  onBlur() {
    if (this.instanciaPipe != null) {
      this.innerValue = this.instanciaPipe.transform(this.innerValue);
    }
    this.onTouchedCallback();
  }

  onFocus() {
    if (this.instanciaPipe != null) {
      this.innerValue = this.instanciaPipe.parse(this.innerValue);
      this.control.updateValueAndValidity();
    }
    this.onTouchedCallback();
    // this.onChangeCallback(this.innerValue);
  }

  writeValue(value: any) {
    if (value !== this.innerValue) {
      this.innerValue = value;

      // hack para que el ControlValueAccesor funcione correctamente cuando está referenciado en componentes con ChangeDetectionStrategy.OnPush
      // (no es nada bonito, pero Angular lleva 2 años con este bug y no lo han solucionado)
      // si no lo incluimos, en algunos casos se corre el riesgo de ir varios valores por detrás en el bit-input
      this.changeDetectorRef.markForCheck();
    }
  }

  registerOnChange(fn: any) {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any) {
    this.onTouchedCallback = fn;
  }

  search(event: any) {
    this.completeMethod.emit(event);
  }

  selectValue(event: any) {
    this.onSelect.emit(event);
  }

  public LOCALE_ES: any = {
    firstDayOfWeek: 1,
    closeText: "Cerrar",
    prevText: "<Ant",
    nextText: "Sig>",
    currentText: "Hoy",
    monthNames: [
      "enero",
      "febrero",
      "marzo",
      "abril",
      "mayo",
      "junio",
      "julio",
      "agosto",
      "septiembre",
      "octubre",
      "noviembre",
      "diciembre"
    ],
    monthNamesShort: [
      "ene",
      "feb",
      "mar",
      "abr",
      "may",
      "jun",
      "jul",
      "ago",
      "sep",
      "oct",
      "nov",
      "dic"
    ],
    dayNames: [
      "domingo",
      "lunes",
      "martes",
      "miércoles",
      "jueves",
      "viernes",
      "sábado"
    ],
    dayNamesShort: ["dom", "lun", "mar", "mié", "jue", "vie", "sáb"],
    dayNamesMin: ["D", "L", "M", "X", "J", "V", "S"],
    weekHeader: "Sm",
    dateFormat: "dd/mm/yy",
    firstDay: 1,
    isRTL: false,
    showMonthAfterYear: false,
    yearSuffix: "",
    timeOnlyTitle: "Elegir una hora",
    timeText: "Hora",
    hourText: "Horas",
    minuteText: "Minutos",
    secondText: "Segundos",
    millisecText: "Milisegundos",
    microsecText: "Microsegundos",
    timezoneText: "Uso horario",
    timeFormat: "HH:mm",
    timeSuffix: "",
    amNames: ["a.m.", "AM", "A"],
    pmNames: ["p.m.", "PM", "P"]
  };

  // decidimos marcar el campo como obligatorio si se ha indicado a la fuerza (por parámetro)
  // o si el control de formulario asociado está saltando por ser obligatorio y no haberse rellenado
  get obligatorio(): boolean {
    return (
      this.forceObligatorio ||
      (this.control && this.control.errors && this.control.errors["required"])
    );
  }

  showAyuda() {
    this.ayudaService.mostrarAyuda(this.ayuda);
  }

  setDisabledState(isDisabled: boolean): void {
    this.disabled = isDisabled;
  }

  get modoLecturaSimple(): boolean {
    // tipo "lectura*", o readOnly no textarea
    return (
      (this.tipo && this.tipo.startsWith("lectura")) ||
      (this.readOnly && this.tipo !== "textarea")
    );
  }

  get modoLecturaMultilinea(): boolean {
    return this.tipo && this.tipo === "textarea" && this.readOnly;
  }
}
