import {Component, 
    OnDestroy}      from '@angular/core';
import {Subscription}   from 'rxjs';
import {MessageService} from '../../shared/service/message.service';
import {Idioma}         from '../../shared/data/idioma';
import {Error}          from '../../shared/data/error';

@Component({   

selector: 'bit-notification-front',
template: `
  <div class="alert alert-danger margin-top-20" role="alert" *ngIf="error"><b>Error:</b> {{error}} <span *ngIf="masInfo">({{masInfo}})</span></div>
`
})
export class BitNotificationFrontComponent implements OnDestroy
{
public error: string;
public masInfo: string;
private subscriptionErrors: Subscription;    

constructor(messageService: MessageService) {
    // Nos suscribimos a la espera de que envíen notificaciones de error
    this.subscriptionErrors = messageService.observableNotificationsErrors.subscribe(
        error => {
            this.showError(error);
    });
    this.subscriptionErrors = messageService.observableErrors.subscribe(
        message => {
            this.showError(message);
    });
}

showError(error: Error) {
    this.error = Idioma.esEspanyolSeleccionado()?error.textES:error.textCA;
    this.masInfo = error.masInfo;
    window.scrollTo(0, 0);
}

cleanError() {
    this.error = null;
}

ngOnDestroy() {
    // Prevent memory leak when component destroyed
    this.error = null;
    this.subscriptionErrors.unsubscribe();
}

}