import {Component, 
    OnDestroy}      from '@angular/core';
import {Subscription}   from 'rxjs';
import {Message}        from 'primeng/components/common/api';

import {MessageService} from '../service/message.service';
import {BitMessage}     from '../data/bitmessage';
import {Error}          from '../data/error';

@Component({   
selector: 'bit-notification',
template: `
  <p-growl [value]="notificationMessages"></p-growl>
  <p-messages [value]="errorMessages" [closable]="true"></p-messages>
`
})
export class BitNotificationComponent implements OnDestroy {
notificationMessages: Message[] = [];
errorMessages: Message[] = [];

public static MENSAJE_VOID: string = "No hi ha dades per mostrar";
private subscriptionNotifications: Subscription;    
private subscriptionNotificationsErrors: Subscription;    

constructor(messageService: MessageService) {
    //avisoService.initAvisosPedientes();  //hacer su propio componente que cargue los avisos
    //console.log("DialogComponent contructor(" + messageService  + ")");
    //Nos suscribimos a la espera de que envíen notificaciones
    this.subscriptionNotifications = messageService.observableNotifications.subscribe(
        message => {
            this.showMessage(message);
    });
    //Nos suscribimos a la espera de que envíen notificaciones de error
    this.subscriptionNotificationsErrors = messageService.observableNotificationsErrors.subscribe(
        notificationError => {
            this.showNotificationError(notificationError);
    });
}

showMessage(message: BitMessage){
    this.notificationMessages = []; 
    this.notificationMessages.push({severity:message.severity, summary: message.title, detail: message.message});
}

showNotificationError(notificationError: Error){
    this.errorMessages = [];
    if (notificationError.multipleText) {
        for (let error of notificationError.multipleText){
            this.errorMessages.push({severity:'error', summary: notificationError.textCA || notificationError.text, detail: error});
        }
    } else {
        this.errorMessages.push({severity:'error', summary: notificationError.textCA ||notificationError.text, detail: notificationError.masInfo});
    }
}

ngOnDestroy() {
    // prevent memory leak when component destroyed
    this.notificationMessages = null;
    this.errorMessages = null;
    this.subscriptionNotifications.unsubscribe();
    this.subscriptionNotificationsErrors.unsubscribe();
}

}