import {
  Component,
  Input,
  OnInit,
  ChangeDetectionStrategy
} from "@angular/core";
import { Router } from "@angular/router";

/**
 * Componenente que muestra la miga de pan en función de los datos de entrada. Estos datos corresponden con
 * un array de objetos que tendrán tres campos: label, url y params. El label debe ser obligatorio y corresponde con la
 * información en pantalla a mostrar. La url y params son opcionales y correposnden con la dirección a la que el usuario podrá ir
 * si pulsa en ella.
 * Ejemplos: [
 *              {label: 'Desenvolupament', url: 'developer/exceptions'},
 *              {label: 'Log'},
 *              {label: 'Usuaris', url: 'mantenimiento/users', params: {query: 'true'}}
 *            ]
 **/

@Component({
  selector: "bit-store-breadcrumb",
  changeDetection: ChangeDetectionStrategy.OnPush,
  template: `
      <ol class="breadcrumb"> 
          <li [class.active]="i == (bc.length - 1)" *ngFor="let path of bc; let i=index">
              <a *ngIf="i < (bc.length - 1) && path.url" (click)="goAddress(path);" href="#">{{path.label}}</a>
              <span *ngIf="i == (bc.length - 1) || !path.url">{{path.label}}</span>
          </li>
      </ol>
`
})
export class BitStoreBreadCrumbComponent implements OnInit {
  @Input("breadcrumb") bc: any[];

  constructor(protected router: Router) {}

  /** Comprobamos si los datos de entrada no están definidos para inicializarlos en blanco */
  ngOnInit(): void {
    if (this.bc == undefined) this.bc = [];
    if (this.bc.length == 0) this.bc.push({ label: " " });
  }

  /** Vamos a la URL que contenga el path */
  goAddress(path: any) {
    if (path.params) {
      this.router.navigate([path.url, path.params]);
    } else {
      this.router.navigate([path.url]);
    }
    return false; //para evitar que navegue al href="#"
  }
}
