import {
  Component,
  Input,
  Output,
  EventEmitter,
  SimpleChanges,
  ChangeDetectionStrategy
} from "@angular/core";

/**
 * Componenente que muestra una barra de botones.
 **/

@Component({
  selector: "bit-store-toolbar",
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: "bit-store-toolbar.component.html"
})
export class BitStoreToolbarComponent {
  public static FILTER_BUTTON: string = "filter";
  public static PRINT_BUTTON: string = "print";
  public static EXPORT_BUTTON: string = "export";
  public static NEW_BUTTON: string = "new";
  public static SAVE_BUTTON: string = "save";
  public static DELETE_BUTTON: string = "delete";
  public static ADD_BUTTON: string = "add";
  public static HELP_BUTTON: string = "help";

  @Input() toolbar: {};
  @Input() ayuda: string; // clave de la ayuda contextual que mostrar para su botón
  @Input() femenino: boolean = false; // género femenino (para los nombres de botones) o no
  @Input() otherActionsDropdown: boolean = false; // determina si las "other actions" deben mostrarse en línea o como un desplegable
  @Output() onToolbarButtonPressed = new EventEmitter<string>();

  mostrarOtherActionsEnDesplegable: boolean;

  enable(button: string): boolean {
    return this.toolbar[button] && this.toolbar[button].enable;
  }

  visible(button: string): boolean {
    return this.toolbar[button] && this.toolbar[button].visible;
  }

  label(button: string, defaultLabel: string): string {
    return (this.toolbar[button] && this.toolbar[button].label) || defaultLabel;
  }

  estilo(button: string, defaultValue: string): string {
    return (
      (this.toolbar[button] && this.toolbar[button].estilo) ||
      defaultValue ||
      "fa fa-balance-spinner"
    );
  }

  showFilter() {
    this.onToolbarButtonPressed.emit(BitStoreToolbarComponent.FILTER_BUTTON);
  }

  print() {
    this.onToolbarButtonPressed.emit(BitStoreToolbarComponent.PRINT_BUTTON);
  }

  exportData() {
    this.onToolbarButtonPressed.emit(BitStoreToolbarComponent.EXPORT_BUTTON);
  }

  newElement() {
    this.onToolbarButtonPressed.emit(BitStoreToolbarComponent.NEW_BUTTON);
  }

  add() {
    this.onToolbarButtonPressed.emit(BitStoreToolbarComponent.ADD_BUTTON);
  }

  save() {
    this.onToolbarButtonPressed.emit(BitStoreToolbarComponent.SAVE_BUTTON);
  }

  delete() {
    this.onToolbarButtonPressed.emit(BitStoreToolbarComponent.DELETE_BUTTON);
  }

  otherAction(button: any) {
    this.onToolbarButtonPressed.emit(button.action);
  }

  /** escuchamos cuando la toolbar cambia para ajustar las other Actions */
  ngOnChanges(changes: SimpleChanges) {
    for (let propName in changes) {
      let changedProp = changes[propName];
      if (propName === "toolbar") {
        let newToolbar = changedProp.currentValue;
        console.log(`nueva toolbar proporcionada ${newToolbar}`);
        this.toolbar = newToolbar;

        // calcular si las other actions deben mostrarse en desplegable
        // (se ha pedido asi y hay más de una)
        let otherActionsVisibles = 0;
        for (let key in this.toolbar) {
          if (this.toolbar[key].otherAction && this.toolbar[key].visible) {
            otherActionsVisibles++;
          }
        }

        this.mostrarOtherActionsEnDesplegable =
          this.otherActionsDropdown && otherActionsVisibles > 1;
      }
    }
  }
}
