import {Component, 
    Input,
    Output,
    ViewChild,
    AfterContentInit,
    EventEmitter}   from '@angular/core';

@Component({
selector: 'bit-upload',
template: `
<div class="row separador"> <!-- *ngIf="simple" -->
    <div class="col-sm-12"  *ngIf="(multiple || filesToUpload.length == 0)">
        <label for="{{id}}" class="btn btn-upload btn-upload-simple">
            {{getSearchLabel()}}&nbsp;&nbsp;<i class="fa fa-cloud-upload"></i>
        </label>
    </div>
    <button type="submit" *ngIf="!uploading && filesToUpload.length != 0" class="btn btn-default btn-upload-simple">Carrega&nbsp;&nbsp;<span class="glyphicon glyphicon-cloud-upload"></span></button>
    <img *ngIf="uploading" width="40" height="40" src="assets/images/load.gif" />
    <input #inputFile type="file" id="{{id}}" name="{{id}}" multiple="{{ multiple ? 'multiple' : null}}" (change)="fileChangeEvent($event)">
    <div class="doclist-upload doclist-upload-simple" *ngIf="filesToUpload.length != 0">
        <div *ngIf="showFiles && filesToUpload != null" class="simple">
            <div class="row row-upload" *ngFor="let file of filesToUpload; let i=index;">
                <div class="col-sm-7">{{file.name}}</div>
                <div class="col-xs-3">{{file.size}} bytes</div>
                <div class="col-sm-2">
                    <button  (click)="removeFile(i)" class="btn btn-default  btn-upload-simple">
                        <span class="fa fa-close"></span>
                    </button>
                </div>
            </div>
        </div>
    </div>
</div>    
`
})
export class BitUploadComponent implements AfterContentInit {

@Input() simple: boolean;  //TODO: quitar esto
@Input() showFiles: boolean;
@Input() id: string = "file";
@Input() multiple: boolean = false;
@Input() private reset: EventEmitter<any>; // se fuerza un reseteo desde fuera
@Input() searchLabel;
@Input() uploading;
@ViewChild('inputFile') inputFile;  //una referencia al input file para resetearlo cuando el usuario elimina un fichero de la lista
filesToUpload: Array<File>; //contiene los ficheros que el usuario ha seleccionado y no ha borrado        
@Output() private onFileChange = new EventEmitter<Array<File>>();

constructor() {
    this.filesToUpload = [];        
}

ngAfterContentInit() {
    this.reset && this.reset.subscribe(x => {
        this.filesToUpload = [];
        this.inputFile.nativeElement.value = ""; //esto lo hacemos para que se dispare el evento (change) si seleccionan un fichero ya seleccionado
        this.onFileChange.emit(this.filesToUpload);
    })
}

/** Se dispara cada vez que el usuario selecciona uno o varios ficheros */
fileChangeEvent(fileInput: any):void {
    let temp = <Array<File>>fileInput.target.files;  //ficheros seleccionados
    for (let i=0;i<temp.length;i++){                 //añadimos los nuevos ficheros seleccionados a la lista   
        this.filesToUpload.push(temp[i]);
    }
    this.onFileChange.emit(this.filesToUpload);      //comunicamos hacia arriba los ficheros seleccionados  
}

/** Se dispara cuando el usuario elimina un fichero de la lista */
removeFile(pos: number): void{
    this.filesToUpload.splice(pos, 1);
    this.inputFile.nativeElement.value = ""; //esto lo hacemos para que se dispare el evento (change) si seleccionan un fichero ya seleccionado
    this.onFileChange.emit(this.filesToUpload);
}

getSearchLabel(): string {
    if (this.searchLabel) {
        return this.searchLabel;
    } else {
        return "Cerca " + (this.multiple ? "fitxer/s" : "fitxer") + " al disc dur";
    }
}

}