import { Component, forwardRef, OnChanges, Input, SimpleChange, ChangeDetectorRef } from "@angular/core";
import { NG_VALUE_ACCESSOR } from "@angular/forms";
import { ActivatedRoute } from "@angular/router";

import { AyudaService } from "../../service/ayuda.service";
import { SessionSharedService } from "../../service/session-shared.service";
import { BitDropDownGeneralComponent } from "./bitdropdown-general.component";

export const CUSTOM_DROPDOWN_MULTIPLE_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => BitDropDownMultipleComponent),
  multi: true
};

/**
 * Clase que utiliza un custom componente para select múltiple de elementos
 * Utiliza la interfaz ControlValueAccessor
 *    <bit-select [domain]="'DOMAIN_TIPO_LOG'" id="tipolog" [control]="formControl"">&nbsp;&nbsp;Tipus d'operació</bit-select>
 *
 * @input data: array de objetos a mostrar en el select. Puede ser null, en cuyo caso domain no puede ser null
 * @input domain: nombre del dominio que contiene los datos y que nos resolverá la página cuando cargue el componente. Puede ser null, en cuyo caso data no puede ser null
 * @input id: identificador del componente select
 **/

@Component({
  selector: "bit-dropdown-multiple",
  templateUrl: "bit-dropdown-multiple.component.html",
  providers: [CUSTOM_DROPDOWN_MULTIPLE_CONTROL_VALUE_ACCESSOR]
})
export class BitDropDownMultipleComponent extends BitDropDownGeneralComponent implements OnChanges {
  @Input() emptyText: string = "Cap";
  @Input() maxSelectedLabels: number = 3;
  @Input() showSeparateSelectedLabels: boolean = false; // indica si se muestran los elementos seleccionados en una columna separada
  @Input() title: string; //título que aparece en el link para eliminar un elemento una vez seleccionado
  // Si se pueden seleccionar muchos elementos, que el label de los elementos seleccionados no
  // aparezca dentro del select, y hacerlo como una lista fuera del select.

  constructor(
    protected router: ActivatedRoute,
    protected sessionShared: SessionSharedService,
    protected ayudaService: AyudaService,
    protected changeDetectorRef: ChangeDetectorRef
  ) {
    super(router, sessionShared, ayudaService, changeDetectorRef);
  }

  ngOnChanges(changes: { [propKey: string]: SimpleChange }) {
    for (let propName in changes) {
      let changedProp = changes[propName];
      if (propName === "data") {
        this.listaOpciones = changedProp.currentValue ? changedProp.currentValue.slice() : [];
        this.postRetrieveData();
      } else if (propName === "dominio") {
        this.domain = changedProp.currentValue;
        // TODO: recargar dominio por si es distinto?
      }
    }
  }

  // custom processing after retrieving data array from service
  postRetrieveData() {}

  get value_lectura(): any {
    let val = this.value;
    let labels: any[] =
      this.listaOpciones && this.listaOpciones.filter(d => val && val.indexOf(d.value) > -1).map(d => d.label);
    return labels ? labels.join(", ") : null;
  }

  labelSelectedItem(id): string {
    return this.listaOpciones.find(c => c.value == id).label;
  }

  deleteSelectedItem(event, item) {
    event.preventDefault();
    let index = this.value.indexOf(item);
    this.value.splice(index, 1);
    // se clona el array. Si no, el texto "{0} elementos seleccionados" del select de Prime no se actualiza.
    this.value = this.value.slice(0);
  }
}
