import { Pipe, PipeTransform } from "@angular/core";

const PADDING = "000000";

@Pipe({ name: "pipeCantidadMoneda" })
export class PipeCantidadMoneda implements PipeTransform {

   private DECIMAL_SEPARATOR: string;
   private DECIMAL_SEPARATOR_INPUT: string;
   private THOUSANDS_SEPARATOR: string;

   constructor() {
      this.DECIMAL_SEPARATOR = ",";
      this.DECIMAL_SEPARATOR_INPUT = ".";
      this.THOUSANDS_SEPARATOR = ".";
   }

   transform(value: number | string, fractionSize: number = 2): string {

      let svalue = String(value);
   
      var regexp = new RegExp('^[1-9]+[0-9]*([.]([0-9]+)?)?$');
      var test = regexp.test(svalue);
      if (!test) {
         return svalue;
      }

      let [ integer, fraction = "" ] = (value || "").toString()
         .split(this.DECIMAL_SEPARATOR_INPUT);

      fraction = fractionSize > 0
         ? this.DECIMAL_SEPARATOR + (fraction + PADDING).substring(0, fractionSize)
         : "";

      integer = integer.replace(/\B(?=(\d{3})+(?!\d))/g, this.THOUSANDS_SEPARATOR);

      return integer + fraction;
   }


   parse(value: string, previous: string, event, fractionSize: number = 2): string {
    
      let svalue = String(value);

      var regexp = new RegExp('^[1-9]+[0-9]*([.][0-9]{3})*(\,([0-9]+)?)?$');
      var test = regexp.test(svalue);   
      if (!test) {
        return svalue;
      }


      let [ integer, fraction = "" ] = (value || "").split(this.DECIMAL_SEPARATOR);

      integer = integer.replace(/\./g,'');


      fraction = parseInt(fraction, 10) > 0 && fractionSize > 0
         ? this.DECIMAL_SEPARATOR_INPUT + (fraction + PADDING).substring(0, fractionSize)
         : "";

      return integer + fraction;

  }

}