import {Injectable}     from '@angular/core';
import {FormGroup}      from '@angular/forms';
import * as moment      from 'moment';
//import moment = require("moment");  versión lógica por la forma en que se exporta moment desde moment.d.ts
import {Subject, Observable} from 'rxjs';
import {UrlSegment}     from '@angular/router';
import {Domain}         from '../data/domain';

@Injectable()
export class GeneralUtils{
    public static FORMAT_SHORT_DATE = "DD/MM/YYYY";    
    public static FORMAT_LONG_DATE = "DD/MM/YYYY HH:mm";
    public static FORMAT_SHORT_TIME = "HH:mm";
    public static FORMAT_SHORT_SECONDS_TIME = "HH:mm:ss";
    private observableLoadingSource = new Subject<string>();        
    public observableLoading;

    constructor(){
        this.observableLoading = this.observableLoadingSource.asObservable();
    }

    public startLoading(): void{
        // console.log("start loading");
        this.observableLoadingSource.next("start");
    }

    public stopLoading(): void{
        // console.log("stop loading");
        this.observableLoadingSource.next("stop");
    }

    public static getTwoDigits(digit: number): string{
        if (digit < 10){
            return '0'+digit;
        }else{
            return ''+digit;
        }
    }

    public static getPathFromURL(segments:UrlSegment[]): string{
        let path = '';
        for (let seg of segments){
            path += seg.path;
            path += '/';
        }
        return path;
    }

    public static buildNombreCompleto(obj: any): string {
        let str = '';
        if(obj.nombre) str += obj.nombre;
        if(obj.apellido1) str += (str.length > 0 ? ' ' : '') + obj.apellido1;
        if(obj.apellido2) str += (str.length > 0 ? ' ' : '') + obj.apellido2;
        return str;
    }
    
    public static getTodayAsString(format: string = GeneralUtils.FORMAT_SHORT_DATE): string{
        return moment(new Date()).format(format);
    }

    public static getDateAsString(fecha: Date, format: string = GeneralUtils.FORMAT_SHORT_DATE): string{
        return moment(fecha).format(format);
    }

    public static getStringAsDate(fecha: string, format: string = GeneralUtils.FORMAT_SHORT_DATE): Date {
        return moment(fecha, format).toDate();
    }
    
    public static getDateTimeAsString(fecha: Date, format: string = GeneralUtils.FORMAT_LONG_DATE): string{
        return moment(fecha).format(format);
    }
    
    public static getTimeAsString(fecha: Date, format: string = GeneralUtils.FORMAT_SHORT_TIME): string{
        return moment(fecha).format(format);
    }

    public static stringToDate(fString:string): Date {
        return moment(fString).toDate();
    }

    public static stringShortToDate(fString:string): Date {
        return moment(fString, "DD/MM/YYYY").toDate();
    }
    
    public static stringToTime(tString:string): Date {
        return moment(tString, "HH:mm").toDate();
    }

    public static getTimeFromDateString(fecha: string): string {
        return moment(fecha).format(GeneralUtils.FORMAT_SHORT_TIME);
    }

    public static getWithDaysOffset(offset: number = 0): string {
        return moment().add(offset, 'days').format(GeneralUtils.FORMAT_SHORT_DATE);
    }

    public static getFirstDayOfCurrentMonth(): string {
        return moment().startOf('month').format(GeneralUtils.FORMAT_SHORT_DATE);        
    }

    public static getLastDayOfCurrentMonth(): string {
        return moment().endOf('month').format(GeneralUtils.FORMAT_SHORT_DATE);
    }

    public static fechaInicioDespuesIgualFechaFin(fechaInicio: string, fechaFin: string): boolean {
        if (fechaInicio && fechaFin) {
            return GeneralUtils.stringShortToDate(fechaInicio) > GeneralUtils.stringShortToDate(fechaFin);
        }
        return false;
    }

    public static horaInicioDespuesHoraFin(horaInicio: string, horaFin: string): boolean {
        if (horaInicio && horaFin) {
            return GeneralUtils.stringToTime(horaInicio) > GeneralUtils.stringToTime(horaFin);
        }
        return false;
    }

    public static fechaEnRango(fecha: string, fechaInicio: string, fechaFin: string): boolean {
        if (fecha) {
            let fechaDate = GeneralUtils.stringShortToDate(fecha);
            let output = ! ( (fechaInicio && (fechaDate < GeneralUtils.stringShortToDate(fechaInicio)))
                              || (fechaFin && (fechaDate > GeneralUtils.stringShortToDate(fechaFin))))
            ;
            return output;
        }
        return false;
    }    

    public static addOpcionVacia(lista: any[], texto?: string) {        
        let nLista = lista.slice();        
        nLista.unshift(new Domain(null, (texto?texto:"")));        
        return nLista;    
    }

    public static getOptionText(list: Domain[], id: number): string {
        for (let d of list) {
            if (+d.value == id) { return d.label; }
        }
        return "-";
    }
}