import {Injectable}   from '@angular/core';
import {Subject}      from 'rxjs';

import {BitMessage}   from '../data/bitmessage';
import {Error}        from '../data/error';
/**
 * 
 * TODO: better modal implementation that doesn't use window.confirm
 */
@Injectable()
export class MessageService {
   // Observable string sources
   private observableMessagesSource = new Subject<BitMessage>(); //mensajes generales que se mostrarán como un dialog en el centro de la pantalla
   private observableNotificationsSource = new Subject<BitMessage>(); //notifiaciones que se mostrarán como un growl
   private observableErrorsSource = new Subject<Error>(); //errores del servidor que se mostrarán como un dialog igual con la traza del error
   private observableNotificacionsErrorsSource = new Subject<Error>(); //errores controlados enviados del servidor que se mostrarán como una ventanita similar al growl
   public  observableMessages;
   public  observableNotifications;
   public  observableErrors;
   public  observableNotificationsErrors;
   

  constructor () {      
      this.observableMessages = this.observableMessagesSource.asObservable();
      this.observableErrors = this.observableErrorsSource.asObservable(); 
      this.observableNotificationsErrors = this.observableNotificacionsErrorsSource.asObservable();
      this.observableNotifications = this.observableNotificationsSource.asObservable();
  }

  /**
   * Ask user to confirm an action. `message` explains the action and choices.
   * Returns promise resolving to `true`=confirm or `false`=cancel
   */
  confirm(message?: string) {
    return new Promise<boolean>(resolve => {
      return resolve(window.confirm(message || ' OK?'));
    });
  };

  /** Enviamos un mensaje. Como mínimo el componente DialogComponent lo recibirá y lo mostrará al usuario */
  sendMessage(message: BitMessage) {    
    this.observableMessagesSource.next(message);
  }

  /** 
   * Enviamos un error. Si el objeto es de tipo notificación se mostrará como una ventana arriba a la derecha con un close para
   * cerrarlo. Si no es de tipo notificación el componente DialogComponent lo recibirá y lo mostrará al usuario en el centro del usuario
   **/
  sendError(error: Error) { 
    //console.log('sendError', error);
    if (error.notificacion){ 
      this.observableNotificacionsErrorsSource.next(error);
    }else{
      this.observableErrorsSource.next(error);
    }
  }

  /** Enviamos una notificación. El componente NotificationComponent lo recibirá y la mostrará en pantalla */
  sendNotification(message: BitMessage){
    this.observableNotificationsSource.next(message);
  }
}