import { Injectable, Inject } from "@angular/core";

import { HttpUtilsService } from "./http-utils.service";

@Injectable()
export class UploadService {
  private UPLOAD_URL: string;
  private DOWNLOAD_URL: string;

  constructor(@Inject("API_URL") private apiUrl: string) {
    this.UPLOAD_URL = `${this.apiUrl}ficheros/upload`;
    this.DOWNLOAD_URL = `${this.apiUrl}ficheros/download/`;
    console.log("UploadService creado con API_URL:", this.apiUrl);
  }

  /**
   * servicio para hacer upload de ficheros al servidor
   * @param url indica la URL a la que queremos subir el fichero. Si es null se utilizará el endpoint por defecto definido en UPLOAD_URL
   * @param files fichero(s) a subir al servidor
   * @param params parámetros adicionales en forma de Map para hacer la subida
   */
  upload(url: string, files: Array<File>, params: { [key: string]: string }) {
    return new Promise((resolve, reject) => {
      let formData: any = new FormData();

      for (let key in params) {
        formData.append(key, params[key]);
      }

      let xhr = new XMLHttpRequest();
      for (let i = 0; i < files.length; i++) {
        formData.append("file", files[i], files[i].name);
      }
      xhr.onreadystatechange = () => {
        if (xhr.readyState === 4) {
          if (xhr.status === 200) {
            resolve(xhr.response);
          } else {
            reject(xhr.response); //TODO: No llega a los componentes
          }
        }
      };
      xhr.open("POST", url == null ? this.UPLOAD_URL : url, true);
      if (HttpUtilsService.LOGIN_AUTHENTICATION && HttpUtilsService.LOGIN_AUTHENTICATION != "") {
        xhr.setRequestHeader("Authorization", "Basic " + HttpUtilsService.LOGIN_AUTHENTICATION);
      }
      xhr.send(formData);
    });
  }

  getUrlDownloadFile(id: number) {
    return `${this.DOWNLOAD_URL}${id}`;
  }
}
