import { Injectable } from "@angular/core";
import { Router } from "@angular/router";
import { Location } from "@angular/common";

import { Action } from "@ngrx/store";
import { Effect, Actions } from "@ngrx/effects";
import * as RouterActions from "../actions/router.action";

import { Observable } from "rxjs";
import { tap, map } from "rxjs/operators";
import { NavigationExtras } from "@angular/router";

@Injectable()
export class RouterEffects {
  constructor(private actions$: Actions, private router: Router, private location: Location) {}

  //este efecto ejecuta la navegación via router escuchando las acciones GO a la que le suministramos la información de donde ir
  @Effect({ dispatch: false })
  navigate$ = this.actions$.ofType(RouterActions.GO).pipe(
    map((action: RouterActions.Go) => action.payload),
    tap(({ path, query: queryParams, extras }) => {
      this.router.navigate(path, { queryParams, ...extras });
    })
  );

  @Effect({ dispatch: false })
  navigateBack$ = this.actions$.ofType(RouterActions.BACK).pipe(tap(() => this.location.back));

  @Effect({ dispatch: false })
  navigateForward$ = this.actions$.ofType(RouterActions.FORWARD).pipe(tap(() => this.location.forward));
}
