import { ActivatedRouteSnapshot, RouterStateSnapshot, Params } from "@angular/router";
import { ActionReducerMap, createFeatureSelector } from "@ngrx/store";

import * as fromRouter from "@ngrx/router-store";
import { MemoizedSelector } from "@ngrx/store";

// representación de la URL en la que nos encontramos como fuente de la verdad
export interface RouterStateUrl {
  url: string;
  queryParams: Params;
  params: Params;
}

// necesitamos incluir routerReducer como state. El paquete @ngrx/router-store nos da la representación del estado
// mediante RouterReducerState
export interface State {
  routerReducer: fromRouter.RouterReducerState<RouterStateUrl>;
}

export const reducers: ActionReducerMap<State> = {
  routerReducer: fromRouter.routerReducer
};

// selector
export const getRouterState = createFeatureSelector<fromRouter.RouterReducerState<RouterStateUrl>>("routerReducer");

/*
 custom serializer que nos permite obtener la representación del estado de la URL donde nos encontramos en cada momento
 Registrando estsa clase conseguiermos que el RouterStore quedará escuchando eventos del Router, por lo que cada vez que naveguemos 
 y accedamos a otra URL, esta clase se ejecutará para obtener esta nueva representación
*/
export class CustomSerializer implements fromRouter.RouterStateSerializer<RouterStateUrl> {
  //Este método nos transforma el estado del router de Angular a nuestra propia estructura
  serialize(routerSate: RouterStateSnapshot): RouterStateUrl {
    const { url } = routerSate; //same as url = routerSate.url;
    const { queryParams } = routerSate.root;

    let state: ActivatedRouteSnapshot = routerSate.root;
    //El router es un árbol de estados sobre el que podemos iterar
    while (state.firstChild) {
      state = state.firstChild;
    }
    const { params } = state;
    //este objeto se linkará a nuestro árbol de estados
    return { url, queryParams, params };
  }
}
